@@ -30,6 +30,7 @@ class ManifestDataset(BaseDataset):
             coordinates (str): 'relative' or 'absolute', indicating the desired format of the bboxes returned.
             dataset_resources (str): disposable resources associated with this dataset
         """
+
         assert dataset_manifest is not None
         assert coordinates in ['relative', 'absolute']
 
@@ -53,7 +54,7 @@ class ManifestDataset(BaseDataset):
         target = image_manifest.labels
         if self.coordinates == 'relative':
             w, h = image.size
-            target = ManifestDataset._box_convert_to_relative(image_manifest.labels, w, h, self.dataset_info)
+            target = ManifestDataset._convert_box_to_relative(image_manifest.labels, w, h, self.dataset_info)
 
         return image, target, str(index)
 
@@ -70,11 +71,12 @@ class ManifestDataset(BaseDataset):
             raise
 
     @staticmethod
-    def _box_convert_to_relative(target, w, h, dataset_info):
+    def _convert_box_to_relative(target, w, h, dataset_info):
         # Convert absolute coordinates to relative coordinates.
         # Example: for image with size (200, 200), (1, 100, 100, 200, 200) => (1, 0.5, 0.5, 1.0, 1.0)
         if dataset_info.type == DatasetTypes.MULTITASK:
-            return {task_name: ManifestDataset._box_convert_to_relative(task_target, w, h, dataset_info.sub_task_infos[task_name]) for task_name, task_target in target.items()}
+            return {task_name: ManifestDataset._convert_box_to_relative(task_target, w, h, dataset_info.sub_task_infos[task_name]) for task_name, task_target in target.items()}
+
         if dataset_info.type == DatasetTypes.OD:
             return [[t[0], t[1] / w, t[2] / h, t[3] / w, t[4] / h] for t in target]
 
@@ -331,3 +333,57 @@ class BoxAlteration:
         logger.log(level, f'[zoom_box] new box {left}, {t}, {r}, {b}, with {w_ratio}, {h_ratio}, out of ({img_w}, {img_h})')
 
         return left, t, r, b
+
+
+class VisionAsImageTextDataset(BaseDataset):
+    """
+    Consume traditional vision datasets of type [DatasetTypes.IC_MULTICLASS, DatasetTypes.IC_MULTILABEL, DatasetTypes.OD], as DatasetTypes.IMAGE_TEXT_MATCHING dataset.
+    For a certain image, negative image-text pairs are generated from the labels that this image does not possess.
+    """
+
+    def __init__(self, dataset: BaseDataset, neg_to_pos_ratio=0, text_aug=None, rnd_seed=0):
+        """
+        Args:
+            dataset: dataset of expected type
+            neg_to_pos_ratio: ratio of negative against positive image text pairs
+            text_aug: a func that augments a string, i.e., a class name, e.g. dog => a photo of dog
+            rnd_seed: random seed for choosing negative class names for negative image text pairs
+        """
+        assert dataset is not None
+        assert dataset.dataset_info.type in [DatasetTypes.IC_MULTICLASS, DatasetTypes.IC_MULTILABEL, DatasetTypes.OD]
+        assert neg_to_pos_ratio >= 0
+        dataset_info = deepcopy(dataset.dataset_info)
+        dataset_info.type = DatasetTypes.IMAGE_TEXT_MATCHING
+
+        super().__init__(dataset_info)
+        self._dataset = dataset
+        self._negative_pair_ratio = neg_to_pos_ratio
+        self._text_aug = text_aug or (lambda x: x)
+        self._rand = random.Random(rnd_seed)
+
+    @property
+    def labels(self):
+        return None
+
+    def __len__(self):
+        return len(self._dataset)
+
+    def _get_single_item(self, index):
+        img, target, _ = self._dataset[index]
+        pos_class_indices = [x[0] for x in target] if self._dataset.dataset_info.type == DatasetTypes.OD else target
+        pos_class_names = [self._dataset.labels[x] for x in pos_class_indices]
+        labels = [(self._text_aug(class_name), 1) for class_name in pos_class_names]
+        if self._negative_pair_ratio > 0:
+            neg_class_indices = set(range(len(self._dataset.labels))) - set(pos_class_indices)
+            neg_class_names = [self._dataset.labels[x] for x in neg_class_indices]
+            if neg_class_names:
+                down_sample_ratio = self._negative_pair_ratio * len(pos_class_names) / len(neg_class_names)
+                if down_sample_ratio < 1:
+                    neg_class_names = [ncn for ncn in neg_class_names if self._rand.random() < down_sample_ratio]
+
+            neg_labels = [(self._text_aug(class_name), 0) for class_name in neg_class_names]
+            labels += neg_labels
+        return img, labels, str(index)
+
+    def close(self):
+        self._dataset.close()
